/***************************************************************************
                  

                          configfile.h 
                          class for config-file handling
                             -------------------
    begin                : Mon June 29 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************

  ABSTRACT:
	- the ctor will read in the whole config-file. Lines beginning with #
	  will be ignored, so they can be used as comment-lines...
	  if another comment-sign shall be used, change COMMENT_SIGN below

	- settings have to be written in the file this way:
	  SETTINGNAME=SETTINGVALUE;
	  where '=' is the default ASSIGN_SIGN and ';' is the default
	  END_SIGN...

	- all found settings will be stored in an associative container, using
	  a std::map<string, string>
	
	- settings can be accessed by using getSetting(). This function is over-
	  loaded. The settings will be returned by storing their values in the
	  |destination|-parameter recieved by reference.
	  The function itself returns TRUE if the setting could be found, other-
	  whise FALSE

 ***************************************************************************/

#ifndef CONFIGFILE_H
#define CONFIGFILE_H


#include <fstream>
using std::fstream;
using std::ios;
using std::streampos;

#include <string>
using std::string;

#include <map>
using std::map;

#include <sstream>
using std::istringstream;
using std::ostringstream;

using namespace std;

const char  ASSIGN_SIGN  	= '=';
const char* END_SIGN     	= ";";	// must be c-string
const char  COMMENT_SIGN 	= '#';


class ConfigFile
{
	public:
		ConfigFile(const string& filename);
		~ConfigFile() {};

		// to get a setting
		bool getSetting(const string& setting_name, long*   destination) const;
		bool getSetting(const string& setting_name, double* destination) const;
		bool getSetting(const string& setting_name, char*   destination) const;

		// set something
		bool set(const string& key, long   value);
		bool set(const string& key, double value);
		bool set(const string& key, const string& value);

	protected:

		fstream m_file;
		map<string, string> m_settings;

		void readSettings();
		streampos searchEntry(const string& text, streampos start_pos = 0);
		void      changeValue(const string& key,  const string& value, streampos entry_pos);
		void      insertEntry(const string& key,  const string& value);

		bool setEntry(const string& key, const string& value)
		/****************************************************************************
		 * creates a new entry or changes an old one
		 * 	RETURN
		 * 		TRUE if old entry could be changed, FALSE if a new one was created
		 ****************************************************************************/
		{
			streampos entry_pos = searchEntry(key);
			if (entry_pos == -1)
			{
				insertEntry( key, value );
				return false;
			}
			else
			{
				changeValue(key, value, entry_pos);
				return true;
			}
		}
};

#endif
