/***************************************************************************
                          bluePong, (C) 2003
                          Thomas Unterthiner                         

                          ponggame.h
                          a simple game of bluePong
                             -------------------
    begin                : Sam Mai 31 2003
    copyright            : (C) 2003 by Thomas Unterthiner
    email                : thomas_unterthiner@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


/*
 * NOTES:
 * - all sprites in the playing-area of the game orientate their position
 *   on the Position of the 4 walls sorrounding the playing-area
 * - wall-enumeration in the PongWall-vector is fixed in a way that
 *   every wall has the same number than the player using it:
 *   0. wall > right
 *   1. wall > left
 *   2. wall > top
 *   3. wall > bottom
 *   (see enum PlayerSide - declaration in pongplayer.h)
 */
 

#ifndef PONGGAME_H
#define PONGGAME_H

#include <vector>
#include <SDL.h>

#include "sdl_functions.h"

#include "pongball.h"
#include "pongpaddle.h"
#include "pongplayer.h"
#include "configfile.h"


// game settings.... 
// damn it, I should put them into some kind of config-file
// somewhen... (-;

class PongGame
{
public: 
	PongGame(SDL_Surface* Surface);
	// ctor
	
	~PongGame();
	// dtor
	
	bool nextTick();
	// makes the next game-movement
	// RETURNS
	// 	FALSE if the game is over, otherwise TRUE

protected:
	short handleCollisionWithWall(ColorSprite& Sprite);
	// checks if another Sprite collides with a wall
	// 	RETURNS
	// 	0 = no collision
	// 	otherwhise: Alignment of the Wall
	
	short handleCollisionWithPaddle(PongBall& Ball);
	// checks if the Ball collides with a Paddle
	// 	RETURNS
	// 	0 = no collision
	// 	otherwhise: Alignment of the Paddle
	
	void ChangeBallDir(Alignment);
	// changes the Direction of the Ball
	
	bool isGameOver();
	// checks if the game is over
	
	void handleInput(PongPlayer* Player);
	// handles the input (moves the paddles) for a specific player
	
	void newPoint();
	// assigns a point to the last player who interacted with the ball.
	// if there isn't such a player, the opponend player (which should 
	// normally be the one on whose side the ball hit the wall) looses a point

	void drawSprites();
	// draws all the sprites

	void resetBall(PongBall* Ball);
	// resets the ball

	void increaseBallSpeed(PongBall* Ball, int By = 1);
	// increases the ball's speed

	inline void drawPlayerPoints(PongPlayer* Player);
	// shows the points a player has on the bar behind the player's wall

	void blinkBar(PongPlayer* Player);
	// let the point-bar of the game-winner blink

	void fillPointSideBar(PongPlayer* Player, Uint32 Color);
	// gets the coordinates of the sidebar behind the wall of a player
	// that is used to display the scores and then fills that part of
	// the Display with a given color

	void setConfigDefaults();
	// set default-values to the config-vars....

private:
	SDL_Surface* mDisplay;
	PongBall* mBall;
	std::vector<PongWall*> mWall;
	std::vector<PongPlayer*> mPlayer;
	std::vector<PongPaddle*> mPaddle;
	ConfigFile mConfig;
	unsigned short mMaxPoints;
	short mLastPlayerWithBall;

	void createWalls();
	// creates the walls, used by the ctor only
	void createPlayers(short PlayerCount);
	// creates the players, used by the ctor only
	//	PARAMETER
	//		short PlayerCount: number of players to be created
	
	void createPaddles(short PaddleCount);
	// creates the paddles, used by the ctor only
	//	PARAMETER
	//		short PaddleCount: number of paddles to be created

	// config-vars:
	// written in capital letters both for historical reasons
	// (they used to be global vars) and for practical reasons
	// (they still are kind of global vars, because they don't
	// change durring 1 session)
	unsigned short DISTANCE_WALL_FROM_HORIZONTAL_BORDER;
	unsigned short DISTANCE_WALL_FROM_VERTICAL_BORDER;
	unsigned short WALL_THICKNESS;
	unsigned short PADDLE_WIDTH;
	unsigned short PADDLE_HEIGHT;
	unsigned short BALL_WIDTH;
	unsigned short BALL_HEIGHT;
	unsigned short BALL_SPEED;
};

#endif
